@@ -10,11 +10,11 @@ Data::Tabulate - Table generation!
 
 =head1 VERSION
 
-Version 0.04
+Version 0.06
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -88,7 +88,9 @@ sub render {
     my @data = @{$atts->{data}};
     my $tmp  = $module;
     $module  = 'Data::Tabulate::Plugin::'.$module;
+    
     $self->_load_module($module);
+    
     @data          = $self->_data() unless @data;
     my @table      = $self->tabulate(@data);
     my $plugin_obj = $module->new();
@@ -128,33 +130,69 @@ sub tabulate {
     
     my $nr   = scalar @data;
     my $cols = int sqrt $nr;
+    
+    # the calculated number of columns should not exceed the maximum
+    # number of columns that the user has specified
     if($cols > $self->max_columns){
         $cols    = $self->max_columns;
     }
     
+    # the calculated number of columns should be greater the minimum
+    # number of columns that the user has specified
     if($cols < $self->min_columns){
         $cols    = $self->min_columns;
     }
     
-    my $rest = ($cols - ($nr % $cols)) % $cols;
-    my $rows = int (($nr + $rest) / $cols);
-    
     $self->{cols} = $cols;
-    $self->{rows} = $rows;
-    $self->{rest} = $rest;
-        
-    my @tmp_data = map{[@data[($_ - $cols + 1).. $_]]}
-                      grep{($_ + 1) % $cols == 0}(0..scalar(@data)-1);
-               
-    if($rest > 0){
-        my $start = $nr - ($cols - $rest);
-        my $end   = $nr - 1;
-        push @tmp_data,[@data[$start..$end],(undef) x $rest];
+    
+    my $index = $cols - 1;
+    
+    # tabulate data
+    my @tmp_data;
+    while ( $index < $nr ) {
+        my $start = $index - $cols + 1;
+        push @tmp_data, [ @data[ $start .. $index ] ];
+        $index += $cols;
     }
                
+    my $fill_value = $self->{fill_value};
+
+    my $rest = ($cols - ($nr % $cols)) % $cols;
+    $self->{rest} = $rest;
+    if($rest > 0){
+        
+        my $start = $nr - ($cols - $rest);
+        my $end   = $nr - 1;
+        
+        push @tmp_data, [
+            @data[$start..$end],
+            ($fill_value) x $rest,
+        ];
+    }
+    
+    $self->{rows} = scalar @tmp_data;
+               
     return @tmp_data;
 }
 
+=head2 fill_with
+
+if the array doesn't provide enough elements the table is filled with 'undef' elements
+sometimes this is not the wanted behaviour. So you can change the value that is used
+to fill the array.
+
+  $obj->fill_with( 'hi' );
+
+for an example see t/04_fill_with.t
+
+=cut
+
+sub fill_with {
+    my ($self,$value) = @_;
+    
+    $self->{fill_value} = $value;
+}
+
 =head2 cols
 
 returns the number of columns the table has
@@ -310,10 +348,10 @@ L<http://search.cpan.org/dist/Data-Tabulate>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Renee Baecker, all rights reserved.
+Copyright 2006 - 2010 Renee Baecker, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+under the same terms of Artistic License 2.0.
 
 =cut
 
